@@ -40,6 +40,7 @@
 #include <libempathy/gossip-conf.h>
 
 #include "gossip-chat-window.h"
+#include "empathy-images.h"
 //#include "gossip-add-contact-dialog.h"
 //#include "gossip-chat-invite.h"
 //#include "gossip-contact-info-dialog.h"
@@ -48,7 +49,6 @@
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
 //#include "gossip-sound.h"
-#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
@@ -106,8 +106,6 @@ struct _GossipChatWindowPriv {
 static void       gossip_chat_window_class_init         (GossipChatWindowClass *klass);
 static void       gossip_chat_window_init               (GossipChatWindow      *window);
 static void       gossip_chat_window_finalize           (GObject               *object);
-static GdkPixbuf *chat_window_get_status_pixbuf         (GossipChatWindow      *window,
-							 GossipChat            *chat);
 static void       chat_window_accel_cb                  (GtkAccelGroup         *accelgroup,
 							 GObject               *object,
 							 guint                  key,
@@ -179,6 +177,7 @@ static void       chat_window_composing_cb              (GossipChat            *
 							 GossipChatWindow      *window);
 static void       chat_window_new_message_cb            (GossipChat            *chat,
 							 GossipMessage         *message,
+							 gboolean               is_backlog,
 							 GossipChatWindow      *window);
 static GtkNotebook* chat_window_detach_hook             (GtkNotebook           *source,
 							 GtkWidget             *page,
@@ -354,9 +353,9 @@ gossip_chat_window_init (GossipChatWindow *window)
 	 * stock image
 	 */
 	image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (priv->menu_conv_info));
-	gtk_image_set_from_stock (GTK_IMAGE (image),
-				  GOSSIP_STOCK_CONTACT_INFORMATION,
-				  GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image),
+				      EMPATHY_IMAGE_CONTACT_INFORMATION,
+				      GTK_ICON_SIZE_MENU);
 
 	/* Set up smiley menu */
 	menu = gossip_chat_view_get_smiley_menu (
@@ -502,30 +501,6 @@ gossip_chat_window_finalize (GObject *object)
 	G_OBJECT_CLASS (gossip_chat_window_parent_class)->finalize (object);
 }
 
-static GdkPixbuf *
-chat_window_get_status_pixbuf (GossipChatWindow *window,
-			       GossipChat       *chat)
-{
-	GossipChatWindowPriv *priv;
-	GdkPixbuf            *pixbuf;
-
-	priv = GET_PRIV (window);
-
-	if (g_list_find (priv->chats_new_msg, chat)) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_MESSAGE,
-					      GTK_ICON_SIZE_MENU);
-	}
-	else if (g_list_find (priv->chats_composing, chat)) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_TYPING,
-					      GTK_ICON_SIZE_MENU);
-	}
-	else {
-		pixbuf = gossip_chat_get_status_pixbuf (chat);
-	}
-
-	return pixbuf;
-}
-
 static void
 chat_window_accel_cb (GtkAccelGroup    *accelgroup,
 		      GObject          *object,
@@ -668,15 +643,24 @@ static void
 chat_window_update_status (GossipChatWindow *window,
 			   GossipChat       *chat)
 {
-	GtkImage  *image;
-	GdkPixbuf *pixbuf;
+	GossipChatWindowPriv *priv;
+	GtkImage             *image;
+	const gchar          *icon_name = NULL;
 
-	pixbuf = chat_window_get_status_pixbuf (window, chat);
+	priv = GET_PRIV (window);
+
+	if (g_list_find (priv->chats_new_msg, chat)) {
+		icon_name = EMPATHY_IMAGE_MESSAGE;
+	}
+	else if (g_list_find (priv->chats_composing, chat)) {
+		icon_name = EMPATHY_IMAGE_TYPING;
+	}
+	else {
+		icon_name = gossip_chat_get_status_icon_name (chat);
+	}
 	image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
-	gtk_image_set_from_pixbuf (image, pixbuf);
+	gtk_image_set_from_icon_name (image, icon_name, GTK_ICON_SIZE_MENU);
 
-	g_object_unref (pixbuf);
-	
 	chat_window_update_title (window, chat);
 	chat_window_update_tooltip (window, chat);
 }
@@ -686,7 +670,6 @@ chat_window_update_title (GossipChatWindow *window,
 			  GossipChat       *chat)
 {
 	GossipChatWindowPriv	*priv;
-	GdkPixbuf		*pixbuf = NULL;
 	const gchar             *str;
 	gchar			*title;
 	gint  			 n_chats;
@@ -742,16 +725,10 @@ chat_window_update_title (GossipChatWindow *window,
 	g_free (title);
 
 	if (priv->chats_new_msg) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_MESSAGE,
-					      GTK_ICON_SIZE_MENU);
+		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
+					  EMPATHY_IMAGE_MESSAGE);
 	} else {
-		pixbuf = NULL;
-	}
-
-	gtk_window_set_icon (GTK_WINDOW (priv->dialog), pixbuf);
-
-	if (pixbuf) {
-		g_object_unref (pixbuf);
+		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 	}
 }
 
@@ -1363,6 +1340,7 @@ chat_window_composing_cb (GossipChat       *chat,
 static void
 chat_window_new_message_cb (GossipChat       *chat,
 			    GossipMessage    *message,
+			    gboolean          is_backlog,
 			    GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
@@ -1382,7 +1360,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 
 	needs_urgency = FALSE;
 	if (gossip_chat_is_group_chat (chat)) {		
-		if (gossip_chat_should_highlight_nick (message)) {
+		if (!is_backlog && 
+		    gossip_chat_should_highlight_nick (message)) {
 			gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
 			needs_urgency = TRUE;
 		}
@@ -1394,7 +1373,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 		chat_window_set_urgency_hint (window, TRUE);
 	}
 
-	if (!g_list_find (priv->chats_new_msg, chat)) {
+	if (!is_backlog && 
+	    !g_list_find (priv->chats_new_msg, chat)) {
 		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
 		chat_window_update_status (window, chat);
 	}
@@ -1500,16 +1480,16 @@ chat_window_page_added_cb (GtkNotebook      *notebook,
 	gossip_chat_set_window (chat, window);
 
 	/* Connect chat signals for this window */
-	g_signal_connect (chat, "status_changed",
+	g_signal_connect (chat, "status-changed",
 			  G_CALLBACK (chat_window_status_changed_cb),
 			  window);
-	g_signal_connect (chat, "name_changed",
+	g_signal_connect (chat, "name-changed",
 			  G_CALLBACK (chat_window_name_changed_cb),
 			  window);
 	g_signal_connect (chat, "composing",
 			  G_CALLBACK (chat_window_composing_cb),
 			  window);
-	g_signal_connect (chat, "new_message",
+	g_signal_connect (chat, "new-message",
 			  G_CALLBACK (chat_window_new_message_cb),
 			  window);
 
@@ -1894,3 +1874,31 @@ gossip_chat_window_has_focus (GossipChatWindow *window)
 
 	return has_focus;
 }
+
+GossipChat *
+gossip_chat_window_find_chat_by_id (const gchar *id)
+{
+	GList *l;
+
+	for (l = chat_windows; l; l = l->next) {
+		GossipChatWindowPriv *priv;
+		GossipChatWindow     *window;
+		GList                *ll;
+
+		window = l->data;
+		priv = GET_PRIV (window);
+
+		for (ll = priv->chats; ll; ll = ll->next) {
+			GossipChat *chat;
+
+			chat = ll->data;
+
+			if (strcmp (id, gossip_chat_get_id (chat)) == 0) {
+				return chat;
+			}
+		}
+	}
+
+	return NULL;
+}
+
